Package: tutorial.helpers
Title: Helper Functions for Creating Tutorials
Version: 0.7.0
Authors@R: 
    person(given = "David",
           family = "Kane",
           role = c("aut", "cre", "cph"),
           email = "dave.kane@gmail.com",
           comment = c(ORCID = "0000-0002-6660-3934"))
Description: Helper functions for creating, editing, and testing tutorials
    created with the 'learnr' package. Provides a simple method for allowing
    students to download their answers to tutorial questions. For examples
    of its use, see the 'r4ds.tutorials' package.
Depends: R (>= 4.1.0)
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: quarto
Suggests: googledrive, quarto, testthat (>= 3.0.0)
Config/testthat/edition: 3
Imports: knitr, learnr, rmarkdown, rstudioapi, rvest, shiny, tibble,
        utils
BugReports: https://github.com/PPBDS/tutorial.helpers/issues
URL: https://ppbds.github.io/tutorial.helpers/,
        https://github.com/PPBDS/tutorial.helpers
NeedsCompilation: no
Packaged: 2026-07-27 01:28:28 UTC; dkane
Author: David Kane [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-6660-3934>)
Maintainer: David Kane <dave.kane@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-27 06:50:09 UTC
