This folder contains actual submissions from students who took Kane's Data Science Bootcamp (https://bootcamp.davidkane.info/) during the summer of 2025. All students answers were submitted to this Google Drive:

https://drive.google.com/drive/folders/1RgxfiwPmcKrhYLOqitLQt4KeODYZEu9ZMVvh029qBfy37V0wEbxIg95gwUE0JWL_hXEBuAlT?usp=sharing

This folder contains all the answers to the regular expressions *data-import* and *stops*. The former corresponds to the Data Import tutorial from the r4ds.tutorials package and the later corresponds to the Stops tutorial from the primer.tutorials package.