## ### Exercise 7
## 
## 
## ```{r plotting-7, exercise = TRUE}
## 
## ```
## 
## <button onclick = "transfer_code(this)">Copy previous code</button>
## 
## ```{r plotting-7-hint, eval = FALSE}
## 
## ```
## 
## ```{r plotting-7-test, include = FALSE}
## 
## ```
## 
## ###

## ### Exercise 9
## 
## ```{r plotting-9}
## question_text(NULL,
## 	answer(NULL, correct = TRUE),
## 	allow_retry = TRUE,
## 	try_again_button = "Edit Answer",
## 	incorrect = NULL,
## 	rows = 3)
## ```
## 
## ###

## ### Exercise 8
## 
## ```{r plotting-8}
## question_text(NULL,
## 	message = "Place correct answer here.",
## 	answer(NULL, correct = TRUE),
## 	allow_retry = FALSE,
## 	incorrect = NULL,
## 	rows = 6)
## ```
## 
## ###
