## Download answers

```{r minutes}
question_numeric(
	"How many minutes, approximately, did it take you to complete this tutorial? For example, an hour and a half would be 90 minutes.",
	answer(NULL, correct = TRUE),
	allow_retry = TRUE,
	try_again = "Edit Answer",
	incorrect = NULL,
  min = 0,
	max = 360,
	step = 1)
```

###

```{r context = "setup"}
tutorial.helpers::submission_ui
```

```{r context = "server"}
tutorial.helpers::submission_server()
```
