---
params:
  title: "Week 0 Tutorial Report"
  output: !r tibble::tibble(id = "test", checked = FALSE, correct = NA)
title: "`r params$title`"
---

```{r setup, include = FALSE}
library(gt)
```

```{r, echo = FALSE, center = TRUE}
gt(params$output)
```

<!-- This file is used in some testing. -->
