## Information
###

```{r name}
question_text(
    "Name:",
    answer(NULL, correct = TRUE),
    allow_retry = TRUE,
    try_again = "Edit Answer",
    incorrect = NULL)
```

```{r email}
question_text(
    "Email:",
    answer(NULL, correct = TRUE),
    allow_retry = TRUE,
    try_again = "Edit Answer",
    incorrect = NULL)
```

```{r ID}
question_text(
    "ID (if requested by your instructor):",
    answer(NULL, correct = TRUE),
    allow_retry = TRUE,
    try_again = "Edit Answer",
    incorrect = NULL)
```
